@@ -1110,6 +1110,9 @@ def tuple_positional_schema(
     Args:
         items_schema: The value must be a tuple with items that match these schemas
         extra_schema: The value must be a tuple with items that match this schema
+            This was inspired by JSON schema's `prefixItems` and `items` fields.
+            In python's `typing.Tuple`, you can't specify a type for "extra" items -- they must all be the same type
+            if the length is variable. So this field won't be set from a `typing.Tuple` annotation on a pydantic model.
         strict: The value must be a tuple with exactly this many items
         ref: See [TODO] for details
         metadata: See [TODO] for details
@@ -1218,7 +1221,9 @@ def set_schema(
         items_schema: The value must be a set with items that match this schema
         min_length: The value must be a set with at least this many items
         max_length: The value must be a set with at most this many items
-        generator_max_length: The value must be a set with at most this many items
+        generator_max_length: At most this many items will be read from a generator before failing validation
+            This is important because generators can be infinite, and even with a `max_length` on the set,
+            an infinite generator could run forever without producing more than `max_length` distinct items.
         strict: The value must be a set with exactly this many items
         ref: See [TODO] for details
         metadata: See [TODO] for details
@@ -1296,6 +1301,7 @@ def frozenset_schema(
 class GeneratorSchema(TypedDict, total=False):
     type: Required[Literal['generator']]
     items_schema: CoreSchema
+    min_length: int
     max_length: int
     ref: str
     metadata: Any
@@ -1305,6 +1311,7 @@ class GeneratorSchema(TypedDict, total=False):
 def generator_schema(
     items_schema: CoreSchema | None = None,
     *,
+    min_length: int | None = None,
     max_length: int | None = None,
     ref: str | None = None,
     metadata: Any = None,
@@ -1325,8 +1332,13 @@ def generator_schema(
     v.validate_python(gen())
     ```
 
+    Unlike other types, validated generators do not raise ValidationErrors eagerly,
+    but instead will raise a ValidationError when a violating value is actually read from the generator.
+    This is to ensure that "validated" generators retain the benefit of lazy evaluation.
+
     Args:
         items_schema: The value must be a generator with items that match this schema
+        min_length: The value must be a generator that yields at least this many items
         max_length: The value must be a generator that yields at most this many items
         ref: See [TODO] for details
         metadata: See [TODO] for details
@@ -1335,6 +1347,7 @@ def generator_schema(
     return dict_not_none(
         type='generator',
         items_schema=items_schema,
+        min_length=min_length,
         max_length=max_length,
         ref=ref,
         metadata=metadata,
@@ -1792,7 +1805,7 @@ def union_schema(
 
 class TaggedUnionSchema(TypedDict, total=False):
     type: Required[Literal['tagged-union']]
-    choices: Required[Dict[str, Union[str, CoreSchema]]]
+    choices: Required[Dict[Union[str, int], Union[str, int, CoreSchema]]]
     discriminator: Required[
         Union[str, List[Union[str, int]], List[List[Union[str, int]]], Callable[[Any], Optional[str]]]
     ]
@@ -1800,19 +1813,21 @@ class TaggedUnionSchema(TypedDict, total=False):
     custom_error_message: str
     custom_error_context: Dict[str, Union[str, int, float]]
     strict: bool
+    from_attributes: bool  # default: True
     ref: str
     metadata: Any
     serialization: SerSchema
 
 
 def tagged_union_schema(
-    choices: Dict[str, str | CoreSchema],
+    choices: Dict[Union[int, str], int | str | CoreSchema],
     discriminator: str | list[str | int] | list[list[str | int]] | Callable[[Any], str | None],
     *,
     custom_error_type: str | None = None,
     custom_error_message: str | None = None,
     custom_error_context: dict[str, int | str | float] | None = None,
     strict: bool | None = None,
+    from_attributes: bool | None = None,
     ref: str | None = None,
     metadata: Any = None,
     serialization: SerSchema | None = None,
@@ -1848,11 +1863,19 @@ def tagged_union_schema(
 
     Args:
         choices: The schemas to match
+            When retrieving a schema from `choices` using the discriminator value, if the value is a str,
+            it should be fed back into the `choices` map until a schema is obtained
+            (This approach is to prevent multiple ownership of a single schema in Rust)
         discriminator: The discriminator to use to determine the schema to use
+            * If `discriminator` is a str, it is the name of the attribute to use as the discriminator
+            * If `discriminator` is a list of int/str, it should be used as a "path" to access the discriminator
+            * If `discriminator` is a list of lists, each inner list is a path, and the first path that exists is used
+            * If `discriminator` is a callable, it should return the discriminator when called on the value to validate
         custom_error_type: The custom error type to use if the validation fails
         custom_error_message: The custom error message to use if the validation fails
         custom_error_context: The custom error context to use if the validation fails
         strict: Whether the underlying schemas should be validated with strict mode
+        from_attributes: Whether to use the attributes of the object to retrieve the discriminator value
         ref: See [TODO] for details
         metadata: See [TODO] for details
         serialization: Custom serialization schema
@@ -1865,6 +1888,7 @@ def tagged_union_schema(
         custom_error_message=custom_error_message,
         custom_error_context=custom_error_context,
         strict=strict,
+        from_attributes=from_attributes,
         ref=ref,
         metadata=metadata,
         serialization=serialization,
@@ -2294,35 +2318,6 @@ def call_schema(
     )
 
 
-class RecursiveReferenceSchema(TypedDict, total=False):
-    type: Required[Literal['recursive-ref']]
-    schema_ref: Required[str]
-    metadata: Any
-    serialization: SerSchema
-
-
-def recursive_reference_schema(
-    schema_ref: str, metadata: Any = None, serialization: SerSchema | None = None
-) -> RecursiveReferenceSchema:
-    """
-    Returns a schema that matches a recursive reference value, e.g.:
-
-    ```py
-    from pydantic_core import SchemaValidator, core_schema
-    schema_recursive = core_schema.recursive_reference_schema('list-schema')
-    schema = core_schema.list_schema(items_schema=schema_recursive, ref='list-schema')
-    v = SchemaValidator(schema)
-    assert v.validate_python([[]]) == [[]]
-    ```
-
-    Args:
-        schema_ref: The schema ref to use for the recursive reference schema
-        metadata: See [TODO] for details
-        serialization: Custom serialization schema
-    """
-    return dict_not_none(type='recursive-ref', schema_ref=schema_ref, metadata=metadata, serialization=serialization)
-
-
 class CustomErrorSchema(TypedDict, total=False):
     type: Required[Literal['custom-error']]
     schema: Required[CoreSchema]
@@ -2555,6 +2550,66 @@ def multi_host_url_schema(
     )
 
 
+class DefinitionsSchema(TypedDict, total=False):
+    type: Required[Literal['definitions']]
+    schema: Required[CoreSchema]
+    definitions: Required[List[CoreSchema]]
+    metadata: Any
+    serialization: SerSchema
+
+
+def definitions_schema(schema: CoreSchema, definitions: list[CoreSchema]) -> DefinitionsSchema:
+    """
+    Build a schema that contains both an inner schema and a list of definitions which can be used
+    within the inner schema.
+
+    ```py
+    from pydantic_core import SchemaValidator, core_schema
+    schema = core_schema.definitions_schema(
+        core_schema.list_schema(core_schema.definition_reference_schema('foobar')),
+        [core_schema.int_schema(ref='foobar')],
+    )
+    v = SchemaValidator(schema)
+    assert v.validate_python([1, 2, '3']) == [1, 2, 3]
+    ```
+
+    Args:
+        schema: The inner schema
+        definitions: List of definitions which can be referenced within inner schema
+    """
+    return DefinitionsSchema(type='definitions', schema=schema, definitions=definitions)
+
+
+class DefinitionReferenceSchema(TypedDict, total=False):
+    type: Required[Literal['definition-ref']]
+    schema_ref: Required[str]
+    metadata: Any
+    serialization: SerSchema
+
+
+def definition_reference_schema(
+    schema_ref: str, metadata: Any = None, serialization: SerSchema | None = None
+) -> DefinitionReferenceSchema:
+    """
+    Returns a schema that points to a schema stored in "definitions", this is useful for nested recursive
+    models and also when you want to define validators separately from the main schema, e.g.:
+
+    ```py
+    from pydantic_core import SchemaValidator, core_schema
+    schema_definition = core_schema.definition_reference_schema('list-schema')
+    schema = core_schema.list_schema(items_schema=schema_definition, ref='list-schema')
+    v = SchemaValidator(schema)
+    assert v.validate_python([[]]) == [[]]
+    ```
+
+    Args:
+        schema_ref: The schema ref to use for the definition reference schema
+        metadata: See [TODO] for details
+        serialization: Custom serialization schema
+    """
+    return dict_not_none(type='definition-ref', schema_ref=schema_ref, metadata=metadata, serialization=serialization)
+
+
 CoreSchema = Union[
     AnySchema,
     NoneSchema,
@@ -2591,11 +2646,12 @@ CoreSchema = Union[
     ModelSchema,
     ArgumentsSchema,
     CallSchema,
-    RecursiveReferenceSchema,
     CustomErrorSchema,
     JsonSchema,
     UrlSchema,
     MultiHostUrlSchema,
+    DefinitionsSchema,
+    DefinitionReferenceSchema,
 ]
 
 # to update this, call `pytest -k test_core_schema_type_literal` and copy the output
@@ -2632,11 +2688,12 @@ CoreSchemaType = Literal[
     'model',
     'arguments',
     'call',
-    'recursive-ref',
     'custom-error',
     'json',
     'url',
     'multi-host-url',
+    'definitions',
+    'definition-ref',
 ]
 
 
