@@ -1,6 +1,8 @@
 #![cfg_attr(has_no_coverage, feature(no_coverage))]
 #![allow(clippy::borrow_deref_ref)]
 
+extern crate core;
+
 use pyo3::prelude::*;
 
 #[cfg(feature = "mimalloc")]
@@ -14,6 +16,7 @@ mod input;
 mod lookup_key;
 mod questions;
 mod recursion_guard;
+mod serializers;
 mod url;
 mod validators;
 
@@ -21,6 +24,7 @@ mod validators;
 pub use self::url::{PyMultiHostUrl, PyUrl};
 pub use build_tools::SchemaError;
 pub use errors::{list_all_errors, PydanticCustomError, PydanticKnownError, PydanticOmit, ValidationError};
+pub use serializers::{to_json, PydanticSerializationError, PydanticSerializationUnexpectedValue, SchemaSerializer};
 pub use validators::SchemaValidator;
 
 pub fn get_version() -> String {
@@ -43,8 +47,12 @@ fn _pydantic_core(_py: Python, m: &PyModule) -> PyResult<()> {
     m.add_class::<PydanticCustomError>()?;
     m.add_class::<PydanticKnownError>()?;
     m.add_class::<PydanticOmit>()?;
-    m.add_class::<self::url::PyUrl>()?;
-    m.add_class::<self::url::PyMultiHostUrl>()?;
+    m.add_class::<PydanticSerializationError>()?;
+    m.add_class::<PydanticSerializationUnexpectedValue>()?;
+    m.add_class::<PyUrl>()?;
+    m.add_class::<PyMultiHostUrl>()?;
+    m.add_class::<SchemaSerializer>()?;
+    m.add_function(wrap_pyfunction!(to_json, m)?)?;
     m.add_function(wrap_pyfunction!(list_all_errors, m)?)?;
     Ok(())
 }
