@@ -1,6 +1,8 @@
 use std::borrow::Cow;
+use std::fmt;
 use std::fmt::Write;
 
+use pyo3::exceptions::PyTypeError;
 use pyo3::intern;
 use pyo3::prelude::*;
 use pyo3::types::{PyDict, PyList, PyString};
@@ -8,7 +10,7 @@ use pyo3::types::{PyDict, PyList, PyString};
 use ahash::AHashMap;
 
 use crate::build_tools::{is_strict, py_err, schema_or_config, SchemaDict};
-use crate::errors::{ErrorType, ValError, ValLineError, ValResult};
+use crate::errors::{ErrorType, LocItem, ValError, ValLineError, ValResult};
 use crate::input::{GenericMapping, Input};
 use crate::lookup_key::LookupKey;
 use crate::questions::Question;
@@ -189,10 +191,53 @@ impl Discriminator {
     }
 }
 
+#[derive(Debug, Clone, Eq, PartialEq, Hash)]
+enum ChoiceKey {
+    Int(i64),
+    Str(String),
+}
+
+impl ChoiceKey {
+    fn from_py(raw: &PyAny) -> PyResult<Self> {
+        if let Ok(py_int) = raw.extract::<i64>() {
+            Ok(Self::Int(py_int))
+        } else if let Ok(py_str) = raw.downcast::<PyString>() {
+            Ok(Self::Str(py_str.to_str()?.to_string()))
+        } else {
+            py_err!(PyTypeError; "Expected int or str, got {}", raw.get_type().name().unwrap_or("<unknown python object>"))
+        }
+    }
+
+    fn repr(&self) -> String {
+        match self {
+            Self::Int(i) => i.to_string(),
+            Self::Str(s) => format!("'{s}'"),
+        }
+    }
+}
+
+impl fmt::Display for ChoiceKey {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        match self {
+            Self::Int(i) => write!(f, "{i}"),
+            Self::Str(s) => write!(f, "{s}"),
+        }
+    }
+}
+
+impl From<&ChoiceKey> for LocItem {
+    fn from(key: &ChoiceKey) -> Self {
+        match key {
+            ChoiceKey::Str(s) => s.as_str().into(),
+            ChoiceKey::Int(i) => (*i).into(),
+        }
+    }
+}
+
 #[derive(Debug, Clone)]
 pub struct TaggedUnionValidator {
-    choices: AHashMap<String, CombinedValidator>,
-    repeat_choices: Option<AHashMap<String, String>>,
+    choices: AHashMap<ChoiceKey, CombinedValidator>,
+    repeat_choices: Option<AHashMap<ChoiceKey, ChoiceKey>>,
     discriminator: Discriminator,
     from_attributes: bool,
     strict: bool,
@@ -216,25 +261,27 @@ impl BuildValidator for TaggedUnionValidator {
 
         let schema_choices: &PyDict = schema.get_as_req(intern!(py, "choices"))?;
         let mut choices = AHashMap::with_capacity(schema_choices.len());
-        let mut repeat_choices_vec: Vec<(String, String)> = Vec::new();
+        let mut repeat_choices_vec: Vec<(ChoiceKey, ChoiceKey)> = Vec::new();
         let mut first = true;
         let mut tags_repr = String::with_capacity(50);
         let mut descr = String::with_capacity(50);
 
         for (key, value) in schema_choices {
-            let tag: String = key.extract()?;
-            if let Ok(py_str) = value.downcast::<PyString>() {
-                let repeat_tag = py_str.to_str()?.to_string();
+            let tag = ChoiceKey::from_py(key)?;
+
+            if let Ok(repeat_tag) = ChoiceKey::from_py(value) {
                 repeat_choices_vec.push((tag, repeat_tag));
                 continue;
             }
+
             let validator = build_validator(value, config, build_context)?;
+            let tag_repr = tag.repr();
             if first {
                 first = false;
-                write!(tags_repr, "'{tag}'").unwrap();
+                write!(tags_repr, "{tag_repr}").unwrap();
                 descr.push_str(validator.get_name());
             } else {
-                write!(tags_repr, ", '{tag}'").unwrap();
+                write!(tags_repr, ", {tag_repr}").unwrap();
                 // no spaces in get_name() output to make loc easy to read
                 write!(descr, ",{}", validator.get_name()).unwrap();
             }
@@ -246,9 +293,10 @@ impl BuildValidator for TaggedUnionValidator {
             let mut wrong_values = Vec::with_capacity(repeat_choices_vec.len());
             let mut repeat_choices = AHashMap::with_capacity(repeat_choices_vec.len());
             for (tag, repeat_tag) in repeat_choices_vec {
-                match choices.get(repeat_tag.as_str()) {
+                match choices.get(&repeat_tag) {
                     Some(validator) => {
-                        write!(tags_repr, ", '{tag}'").unwrap();
+                        let tag_repr = tag.repr();
+                        write!(tags_repr, ", {tag_repr}").unwrap();
                         write!(descr, ",{}", validator.get_name()).unwrap();
                         repeat_choices.insert(tag, repeat_tag);
                     }
@@ -265,7 +313,7 @@ impl BuildValidator for TaggedUnionValidator {
         };
 
         let key = intern!(py, "from_attributes");
-        let from_attributes = schema_or_config(schema, config, key, key)?.unwrap_or(false);
+        let from_attributes = schema_or_config(schema, config, key, key)?.unwrap_or(true);
 
         let descr = match discriminator {
             Discriminator::SelfSchema => "self-schema".to_string(),
@@ -304,10 +352,10 @@ impl Validator for TaggedUnionValidator {
                         // errors when getting attributes which should be "raised"
                         match lookup_key.$get_method($( $dict ),+)? {
                             Some((_, value)) => {
-                                if self.strict {
-                                    value.strict_str()
+                                if let Ok(int) = value.validate_int(self.strict) {
+                                    Ok(ChoiceKey::Int(int))
                                 } else {
-                                    value.lax_str()
+                                    Ok(ChoiceKey::Str(value.validate_str(self.strict)?.as_cow()?.as_ref().to_string()))
                                 }
                             }
                             None => Err(self.tag_not_found(input)),
@@ -321,20 +369,20 @@ impl Validator for TaggedUnionValidator {
                     GenericMapping::PyMapping(mapping) => find_validator!(py_get_mapping_item, mapping),
                     GenericMapping::JsonObject(mapping) => find_validator!(json_get, mapping),
                 }?;
-                self.find_call_validator(py, tag.as_cow()?, input, extra, slots, recursion_guard)
+                self.find_call_validator(py, &tag, input, extra, slots, recursion_guard)
             }
             Discriminator::Function(ref func) => {
                 let tag = func.call1(py, (input.to_object(py),))?;
                 if tag.is_none(py) {
                     Err(self.tag_not_found(input))
                 } else {
-                    let tag: &PyString = tag.downcast(py)?;
-                    self.find_call_validator(py, tag.to_string_lossy(), input, extra, slots, recursion_guard)
+                    let tag: &PyAny = tag.downcast(py)?;
+                    self.find_call_validator(py, &(ChoiceKey::from_py(tag)?), input, extra, slots, recursion_guard)
                 }
             }
             Discriminator::SelfSchema => self.find_call_validator(
                 py,
-                self.self_schema_tag(py, input)?,
+                &ChoiceKey::Str(self.self_schema_tag(py, input)?.into_owned()),
                 input,
                 extra,
                 slots,
@@ -407,23 +455,23 @@ impl TaggedUnionValidator {
     fn find_call_validator<'s, 'data>(
         &'s self,
         py: Python<'data>,
-        tag: Cow<str>,
+        tag: &ChoiceKey,
         input: &'data impl Input<'data>,
         extra: &Extra,
         slots: &'data [CombinedValidator],
         recursion_guard: &'s mut RecursionGuard,
     ) -> ValResult<'data, PyObject> {
-        if let Some(validator) = self.choices.get(tag.as_ref()) {
+        if let Some(validator) = self.choices.get(tag) {
             return match validator.validate(py, input, extra, slots, recursion_guard) {
                 Ok(res) => Ok(res),
-                Err(err) => Err(err.with_outer_location(tag.as_ref().into())),
+                Err(err) => Err(err.with_outer_location(tag.into())),
             };
         } else if let Some(ref repeat_choices) = self.repeat_choices {
-            if let Some(choice_tag) = repeat_choices.get(tag.as_ref()) {
+            if let Some(choice_tag) = repeat_choices.get(tag) {
                 let validator = &self.choices[choice_tag];
                 return match validator.validate(py, input, extra, slots, recursion_guard) {
                     Ok(res) => Ok(res),
-                    Err(err) => Err(err.with_outer_location(tag.as_ref().into())),
+                    Err(err) => Err(err.with_outer_location(tag.into())),
                 };
             }
         }
