@@ -3,7 +3,15 @@ from __future__ import annotations as _annotations
 from datetime import date, datetime, time
 from typing import Any
 
-from pydantic_core import PydanticKnownError, SchemaError, SchemaSerializer, SchemaValidator, core_schema
+from pydantic_core import (
+    ErrorDetails,
+    PydanticKnownError,
+    SchemaError,
+    SchemaSerializer,
+    SchemaValidator,
+    ValidationError,
+    core_schema,
+)
 from pydantic_core.core_schema import CoreConfig, CoreSchema
 
 
@@ -69,8 +77,7 @@ def test_schema_typing() -> None:
         'schema': {'type': 'typed-dict', 'return_fields_set': True, 'fields': {'bar': {'schema': {'type': 'str'}}}},
     }
     SchemaValidator(schema)
-    # waiting for https://github.com/microsoft/pyright/issues/4313 to be fixed
-    schema: CoreSchema = {  # type: ignore
+    schema: CoreSchema = {
         'type': 'typed-dict',
         'fields': {
             'a': {'schema': {'type': 'str'}},
@@ -94,7 +101,7 @@ def test_schema_typing() -> None:
                     'type': 'default',
                     'schema': {
                         'type': 'union',
-                        'choices': [{'type': 'none'}, {'type': 'recursive-ref', 'schema_ref': 'Branch'}],
+                        'choices': [{'type': 'none'}, {'type': 'definition-ref', 'schema_ref': 'Branch'}],
                     },
                     'default': None,
                 }
@@ -208,3 +215,17 @@ def test_ser_function_wrap():
         "SerializationInfo(include=None, exclude=None, mode='json', by_alias=True, exclude_unset=False, "
         'exclude_defaults=False, exclude_none=False, round_trip=False)'
     )
+
+
+def test_error_details() -> None:
+    # Test that the ErrorDetails type is correctly exported.
+    def act_on_error_details(_: ErrorDetails) -> None:
+        pass
+
+    v = SchemaValidator({'type': 'int'})
+
+    try:
+        v.validate_python('not an int')
+    except ValidationError as err:
+        for details in err.errors():
+            act_on_error_details(details)
