@@ -36,10 +36,10 @@ def test_lax_bytes_validator():
         v.validate_python('🐈 Hello \ud800World')
     assert exc_info.value.errors() == [
         {
-            'kind': 'string_unicode',
-            'loc': [],
-            'message': 'Input should be a valid string, unable to parse raw data as a unicode string',
-            'input_value': '🐈 Hello \ud800World',
+            'type': 'string_unicode',
+            'loc': (),
+            'msg': 'Input should be a valid string, unable to parse raw data as a unicode string',
+            'input': '🐈 Hello \ud800World',
         }
     ]
 
@@ -101,11 +101,11 @@ def test_length_ctx():
         v.validate_python(b'1')
     assert exc_info.value.errors() == [
         {
-            'kind': 'bytes_too_short',
-            'loc': [],
-            'message': 'Data should have at least 2 bytes',
-            'input_value': b'1',
-            'context': {'min_length': 2},
+            'type': 'bytes_too_short',
+            'loc': (),
+            'msg': 'Data should have at least 2 bytes',
+            'input': b'1',
+            'ctx': {'min_length': 2},
         }
     ]
 
@@ -114,10 +114,10 @@ def test_length_ctx():
 
     assert exc_info.value.errors() == [
         {
-            'kind': 'bytes_too_long',
-            'loc': [],
-            'message': 'Data should have at most 3 bytes',
-            'input_value': b'1234',
-            'context': {'max_length': 3},
+            'type': 'bytes_too_long',
+            'loc': (),
+            'msg': 'Data should have at most 3 bytes',
+            'input': b'1234',
+            'ctx': {'max_length': 3},
         }
     ]
