@@ -122,7 +122,7 @@ def test_ask():
 
     v = SchemaValidator(
         {
-            'type': 'new-class',
+            'type': 'model',
             'cls': MyModel,
             'schema': {
                 'type': 'chain',
