@@ -30,11 +30,11 @@ def test_function_before_raise():
     # debug(str(exc_info.value))
     assert exc_info.value.errors() == [
         {
-            'kind': 'value_error',
-            'loc': [],
-            'message': 'Value error, foobar',
-            'input_value': 'input value',
-            'context': {'error': 'foobar'},
+            'type': 'value_error',
+            'loc': (),
+            'msg': 'Value error, foobar',
+            'input': 'input value',
+            'ctx': {'error': 'foobar'},
         }
     ]
 
@@ -52,11 +52,11 @@ def test_function_before_error():
         v.validate_python('12345')
     assert exc_info.value.errors() == [
         {
-            'kind': 'string_too_long',
-            'loc': [],
-            'message': 'String should have at most 5 characters',
-            'input_value': '12345x',
-            'context': {'max_length': 5},
+            'type': 'string_too_long',
+            'loc': (),
+            'msg': 'String should have at most 5 characters',
+            'input': '12345x',
+            'ctx': {'max_length': 5},
         }
     ]
     assert repr(exc_info.value).startswith('1 validation error for function-before[my_function(), constrained-str]\n')
@@ -82,11 +82,11 @@ def test_function_before_error_model():
         v.validate_python({'my_field': '12345'})
     assert exc_info.value.errors() == [
         {
-            'kind': 'string_too_long',
-            'loc': ['my_field'],
-            'message': 'String should have at most 5 characters',
-            'input_value': '12345x',
-            'context': {'max_length': 5},
+            'type': 'string_too_long',
+            'loc': ('my_field',),
+            'msg': 'String should have at most 5 characters',
+            'input': '12345x',
+            'ctx': {'max_length': 5},
         }
     ]
 
@@ -143,10 +143,10 @@ def test_wrap_error():
         v.validate_python('wrong')
     assert exc_info.value.errors() == [
         {
-            'kind': 'int_parsing',
-            'loc': [],
-            'message': 'Input should be a valid integer, unable to parse string as an integer',
-            'input_value': 'wrong',
+            'type': 'int_parsing',
+            'loc': (),
+            'msg': 'Input should be a valid integer, unable to parse string as an integer',
+            'input': 'wrong',
         }
     ]
 
@@ -163,10 +163,10 @@ def test_function_wrap_location():
     # insert_assert(exc_info.value.errors())
     assert exc_info.value.errors() == [
         {
-            'kind': 'int_parsing',
-            'loc': ['foo'],
-            'message': 'Input should be a valid integer, unable to parse string as an integer',
-            'input_value': 'wrong',
+            'type': 'int_parsing',
+            'loc': ('foo',),
+            'msg': 'Input should be a valid integer, unable to parse string as an integer',
+            'input': 'wrong',
         }
     ]
 
@@ -323,7 +323,7 @@ def test_class_with_validator():
         v.validate_python(True)
 
     assert exc_info.value.errors() == [
-        {'kind': 'string_type', 'loc': [], 'message': 'Input should be a valid string', 'input_value': True}
+        {'type': 'string_type', 'loc': (), 'msg': 'Input should be a valid string', 'input': True}
     ]
 
 
@@ -338,11 +338,11 @@ def test_raise_assertion_error():
 
     assert exc_info.value.errors() == [
         {
-            'kind': 'assertion_error',
-            'loc': [],
-            'message': 'Assertion failed, foobar',
-            'input_value': 'input value',
-            'context': {'error': 'foobar'},
+            'type': 'assertion_error',
+            'loc': (),
+            'msg': 'Assertion failed, foobar',
+            'input': 'input value',
+            'ctx': {'error': 'foobar'},
         }
     ]
 
@@ -358,11 +358,11 @@ def test_raise_assertion_error_plain():
 
     assert exc_info.value.errors() == [
         {
-            'kind': 'assertion_error',
-            'loc': [],
-            'message': 'Assertion failed, Unknown error',
-            'input_value': 'input value',
-            'context': {'error': 'Unknown error'},
+            'type': 'assertion_error',
+            'loc': (),
+            'msg': 'Assertion failed, Unknown error',
+            'input': 'input value',
+            'ctx': {'error': 'Unknown error'},
         }
     ]
 
