@@ -1,6 +1,5 @@
 import platform
 import re
-import sys
 from collections import deque
 from collections.abc import Sequence
 from typing import Any, Dict
@@ -318,7 +317,6 @@ def my_sequence():
     return MySequence
 
 
-@pytest.mark.xfail(sys.platform == 'win32', reason='TODO: why doesnt `validate_python` raise an error on windows here?')
 def test_sequence(MySequence):
     v = SchemaValidator({'type': 'list', 'items_schema': {'type': 'int'}})
     with pytest.raises(ValidationError) as exc_info:
